//Original by Timeslip //mods by harlanrm
Matrix TexProj;
Matrix worldviewproj;
Matrix viewproj;
Matrix world;
Matrix world2;
float3 SunVec;
float3 SunCol;
float3 EyePos;
float3 SunAmb;
float3 SkyCol;
float ticks;
float2 offset;
float TexBlend;
float2 rcpres;
int FogCol;
float3 FogCol2;
float FogStart;
float FogRange;
float GrassDist;
float2 WindVec;

texture tex0;
texture tex1;
texture tex2;
texture tex3;
sampler s0 = sampler_state { texture = <tex0>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=clamp; addressv = clamp; };
sampler s1 = sampler_state { texture = <tex1>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s2 = sampler_state { texture = <tex2>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s3 = sampler_state { texture = <tex3>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=wrap; addressv = wrap; };
sampler s4 = sampler_state { texture = <tex3>; minfilter = linear; magfilter = linear; mipfilter = linear; addressu=clamp; addressv = clamp; };

//For sampling the far water texture
sampler s5 = sampler_state { texture = <tex1>; minfilter = linear; magfilter = linear; mipfilter = none; addressu=wrap; addressv = wrap; };
sampler s6 = sampler_state { texture = <tex2>; minfilter = linear; magfilter = linear; mipfilter = none; addressu=wrap; addressv = wrap; };

//static const float3 watercolour = { 0.9, 0.9, 1 };//original //wasn't used previously

static const float3 watercolour = { 0.4, 0.45, 0.5 };

/*
static const float4x4 TexProj = {
	0.5, 0.0, 0.0, 0.5,
	0.0, 0.5, 0.0, 0.5,
	0.0, 0.0, 0.5, 0.5,
	0.0, 0.0, 0.0, 1.0
};
*/

float4 RenderShader( in float2 TexCoords : TEXCOORD0, in float3 Light : TEXCOORD1, in float4 Height : TEXCOORD2) : COLOR0
{
	clip(Height);
	float3 Result = tex2D( s3, TexCoords) * Light;
	return float4(Result, 1);
}

struct StatVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
};

struct StatVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float fog : TEXCOORD3;
};

StatVertOut StaticVS( StatVertIn IN ) {
	StatVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, world2);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Projection
	OUT.pos = mul(worldpos, viewproj);
	
	//Lighting
	OUT.normal = normalize( IN.normal * 2 - 1 ); //Decompress
	OUT.normal = normalize( mul(OUT.normal, (float3x3)world2) ); //Rotate normals but don't translate them
	OUT.color = IN.color;
	
	return OUT;
}

struct StatInstVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
	float4 m1 : TEXCOORD1;
	float4 m2 : TEXCOORD2;
	float4 m3 : TEXCOORD3;
	float4 m4 : TEXCOORD4;
};

StatVertOut StaticInstVS( StatInstVertIn IN ) {
	StatVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Projection
	OUT.pos = mul(worldpos, viewproj);
	
	//Lighting
	OUT.normal = normalize( mul(IN.normal, (float3x3)worldmat) );
	OUT.color = IN.color;
	
	return OUT;
}

float4 StaticPS( StatVertOut IN ) : COLOR0 {
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - 0.5);
	
	//Lighting
	//float3 diffuse = saturate(dot(IN.normal,-SunVec)) * SunCol;
	//float3 ambient = SunAmb;
	
	//Result.rgb *= (diffuse + ambient) * IN.color;
	
	//Lighting
	float3 normal = normalize(IN.normal);
	Result.rgb *= saturate( (saturate(dot(-SunVec,normal)) * SunCol * IN.color) + (SunAmb * IN.color) );
	
	//Fogging
	Result.rgb = lerp(FogCol2, Result.rgb, IN.fog);

	return float4(Result.rgb, 1);
}

struct StatRefVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float fog : TEXCOORD3;
	float height : TEXCOORD4;
};

StatRefVertOut StaticReflectionVS( StatVertIn IN ) {
	StatRefVertOut OUT;
	
	Matrix wvp = mul(world2, viewproj);
	OUT.pos = mul(IN.pos, wvp);
	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, world2);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Lighting
	OUT.normal = normalize( IN.normal * 2 - 1 ); //Decompress
	OUT.normal = normalize( mul(OUT.normal, (float3x3)world2) ); //Rotate normals but don't translate them
	OUT.color = IN.color;
	
	//Figure hight of pixel in world coordinates
	OUT.height = worldpos.z;
	
	return OUT;
}

StatRefVertOut StaticReflectionInstVS( StatInstVertIn IN ) {
	StatRefVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	OUT.fog = saturate( (FogRange - length(worldpos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Projection
	OUT.pos = mul(worldpos, viewproj);
	
	//Lighting
	OUT.normal = normalize( mul(IN.normal, (float3x3)worldmat) );
	OUT.color = IN.color;
		
	//Figure height of pixel in world coordinates
	OUT.height = worldpos.z;
	
	return OUT;
}

float4 StaticReflectionPS( StatRefVertOut IN ) : COLOR0 {
	//Clip any pixels below a world height of zero
	clip(IN.height);
	
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - 0.5);
	
	//Lighting
	float3 normal = normalize(IN.normal);
	Result.rgb *= saturate( (saturate(dot(-SunVec,normal)) * SunCol * IN.color) + (SunAmb * IN.color) );

	//Fogging
	Result.rgb = lerp(SkyCol, Result.rgb, IN.fog);

	return float4(Result.rgb, 1);
}

struct LowestLandVertOut {
	float4 position : POSITION;
	float4 pos : TEXCOORD0;
	float2 texcoords : TEXCOORD1;
	float3 light : COLOR0;
};

LowestLandVertOut LowestLandVS( in float4 pos : POSITION ) {
	LowestLandVertOut OUT;
	
	float4 modpos = float4(pos.xy, -2080, pos.w);
	
	OUT.position = mul( modpos, worldviewproj);
	
	OUT.pos = float4(modpos.xy - offset, modpos.zw);
	
	//Calculate texture coordinates
	OUT.texcoords = OUT.pos.xy/600;
	
	//Lighting
	float3 normal = float3(0,0,1);
	
	OUT.light = dot(normal,-SunVec) * SunCol;
	
	OUT.light = saturate(OUT.light) + saturate(SunAmb);
	
	return OUT;
}

float4 LowestLandPS( in LowestLandVertOut IN ) : COLOR0 {  
	//Define % to Fog
	float fog = saturate( (FogRange - length(IN.pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	float3 Result = tex2D( s3, IN.texcoords).rgb;
	Result *= IN.light;
	
	//Fogging
	Result = lerp(FogCol2, Result, fog);
	
	return float4(Result, 1);
}

struct WaterVertOut {
	float4 position : POSITION;
	float4 pos : TEXCOORD0;
	float2 texcoord1 : TEXCOORD2;
	float2 texcoord2 : TEXCOORD3;
	float2 texcoord3 : TEXCOORD4;
	float4 screenpos : TEXCOORD5;
};

WaterVertOut WaterVS( in float4 pos : POSITION ) {
	WaterVertOut OUT;
	
	OUT.position = mul(pos, worldviewproj);
	
	OUT.screenpos = mul( pos, TexProj );
	
	OUT.pos = float4(pos.xy - offset, pos.zw);
	
	//Calculate various texture coordinates
	OUT.texcoord1 = OUT.pos.xy/6127;
	OUT.texcoord2 = OUT.pos.xy/1472;
	OUT.texcoord3 = OUT.pos.xy/892;
		
	return OUT;
}
float3 GetWaterNormal( in float2 texcoords, float blend ) : NORMAL {
	float3 normal = lerp( tex2D(s5,texcoords).rgb, tex2D(s6,texcoords).rgb, float3(blend,blend,blend) );
	return normalize( normal * 2 - 1 );
}

float3 GetFinalWaterNormal( float2 texcoord1, float2 texcoord2, float dist ) : NORMAL {
	//Blend together the normals from different sized areas of the same texture.
	float3 far_normal = GetWaterNormal( texcoord1, TexBlend );
	float3 close_normal = GetWaterNormal( texcoord2, TexBlend );
		                      
	return normalize( lerp( normalize(close_normal), normalize(far_normal), saturate(dist/8000) ) );
}

float4 ReflectionShader2( in WaterVertOut IN ) : COLOR0
{        
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
   
   float3 normal = GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   //float3 normal = float3(0, 0, 1 );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal.xy * 3;
   
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
      
   //fresnel factor
   float fresnel = dot(EyeVec,normal);

   //Sample the reflection texture   
   float3 Result = tex2Dproj( s0, IN.screenpos + float4(perturb*6,0,0) ) / 3;
   
   //Sample refraction texture
   float3 Refract = lerp( tex2Dproj(s4, IN.screenpos + float4(perturb.yx,0,0)), watercolour, 0.3);
      
   Result = lerp( Result, Refract, fresnel );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(normal,-SunVec)) * Result * SunCol) + (Result * SunAmb);

   //Result *= watercolour;

   //Specular lighting
   float reflection = dot(EyeVec,reflect(SunVec,normal));
   float3 Specular = pow(reflection,40) * SunCol;

   //Additional Specular
   //Specular += pow(reflection,1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result = Result + Specular / 2;

   //Fogging
   Result = lerp(FogCol2, Result, fog);

   return float4(Result, 1);
}

float4 NonReflectionShader( in WaterVertOut IN ) : COLOR0
{        
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
   
   float3 normal = GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal.xy * 3;
   
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
      
   //fresnel factor
   float fresnel = dot(EyeVec,normal);

   //Assume the sky color is the reflection color 
   float3 Result = SkyCol / 3;
   
   //Sample refraction texture
   float3 Refract = lerp( tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)), watercolour, 0.3);
      
   Result = lerp( Result, Refract, fresnel );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(normal,-SunVec)) * Result * SunCol) + (Result * SunAmb);

   //Result *= watercolour;

   //Specular lighting
   float reflection = dot(EyeVec,reflect(SunVec,normal));
   float3 Specular = pow(reflection,40) * SunCol;

   //Additional Specular
   //Specular += pow(reflection,1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result = Result + Specular / 2;

   //Fogging
   Result = lerp(FogCol2, Result, fog);

   return float4(Result, 1);
}

float4 UnderWaterPS( in WaterVertOut IN ) : COLOR0
{
   //Define % to Fog
   float dist = length(IN.pos.xyz - EyePos.xyz);
   float fog = saturate( ( FogRange - dist ) / (FogRange-FogStart) );
   
   float3 normal = -GetFinalWaterNormal( IN.texcoord1, IN.texcoord2, dist );
   
   //Base the perturbation on the x and y direction of the normal
   float2 perturb = normal.xy * 5;
   
   //Calculate eye vector
   float3 EyeVec = normalize(EyePos - IN.pos.xyz);
      
   //Fresnel Factor
   //float fresnel = 1 - pow( 1 -dot(EyeVec,normal), 2);
   float fresnel = dot(EyeVec,normal);
         
   //Assume the sky color is the reflection color 
   float3 Result = SkyCol / 3;
   
   //Sample refraction texture
   float3 Refract = lerp( tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)), watercolour, 0.2);
      
   Result = lerp( Result, Refract, fresnel );
   
   //Ambient and Diffuse lighting
   Result = (saturate(dot(-normal,-SunVec)) * SunCol * Result) + (Result * (SunAmb + watercolour));

   //Result *= watercolour;

   //Specular lighting
   float3 Specular = pow(dot(EyeVec,reflect(SunVec,normal)),40) * SunCol;

   //Additional Specular
   //Specular += pow(dot(EyeVec,reflect(SunVec,normal)),1000)*(1+4*fog);
   Specular *= pow((SunCol+SunCol.g-SunCol.b)+0.3,3);
   
   Result += Specular * 4;

   //Fogging
   Result = lerp(FogCol2, Result, fog);

   return float4(Result, 1);
   
   //return float4(tex2Dproj(s4, IN.screenpos + float4(perturb,0,0)).rgb, 1);
}

struct LandVertOut {
	float4 pos: POSITION;
	float2 texcoord: TEXCOORD1;
	float3 position: TEXCOORD2;
};
LandVertOut LandscapeVertexShader( float4 pos: POSITION, float2 texcoord: TEXCOORD0 ) {
	LandVertOut OUT;

	//Lower vertices that are close to the camera to ensure they are always below it
	float dist = length(pos.xyz - EyePos.xyz);
	float disp = lerp( 200, 0, saturate(dist/2000));
	pos.z -= disp;
	
	OUT.pos = mul(pos, worldviewproj);
	OUT.texcoord = texcoord;
	OUT.position = pos;
	
	//Define % to Fog
	//OUT.fog = saturate( (FogRange - length(pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );

	return OUT;
}
float4 LandscapePixelShader( in LandVertOut IN ) : COLOR0 {
	//Normalize incoming normal
	float3 normal = normalize(tex2D(s1,IN.texcoord).rgb * 2 - 1);
	
	//World Texture
	float3 color = tex2D(s3,IN.texcoord).rgb;
	
	//Detail Texture
	float detail = tex2D(s2,IN.texcoord * 1064).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 333).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 90).g + 0.5;
		
	//Lighting
	color *= detail * saturate( (saturate(dot(-SunVec,normal)) * SunCol) + SunAmb );
	
	//Define % to Fog
	float fog = saturate( (FogRange - length(IN.position.xyz - EyePos.xyz) ) / (FogRange-FogStart) );

	color = lerp(FogCol2, color, fog);
	
	return float4(color,1);
}

struct LandRefVertOut {
	float4 pos: POSITION;
	float2 texcoord: TEXCOORD1;
	float fog: TEXCOORD2;
	float height: TEXCOORD3;
};
LandRefVertOut LandscapeRefVS( float4 pos: POSITION, float2 texcoord: TEXCOORD0 ) {
	LandRefVertOut OUT;

	//Lower vertices so there are fewer seams visible in the reflection
	pos.z -= 100;
	
	OUT.pos = mul(pos, worldviewproj);
	OUT.texcoord = texcoord;
	
	//Define % to Fog
	OUT.fog = saturate( (FogRange - length(pos.xyz - EyePos.xyz) ) / (FogRange-FogStart) );
	
	//Define height of pixel for clipping to water plane
	OUT.height = pos.z;

	return OUT;
}
float4 LandscapeRefPS( in LandRefVertOut IN ) : COLOR0 {
	//Clip to water plane
	clip(IN.height);

	//Normalize incoming normal
	float3 normal = normalize(tex2D(s1,IN.texcoord).rgb * 2 - 1);
	
	//World Texture
	float3 color = tex2D(s3,IN.texcoord).rgb;
	
	//Detail Texture
	float detail = tex2D(s2,IN.texcoord * 1064).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 333).g + 0.5;
	detail *= tex2D(s2,IN.texcoord * 90).g + 0.5;
	
	//Lighting
	color *= detail * saturate( (saturate(dot(-SunVec,normal)) * SunCol) + SunAmb );
	
	//Fogging
	color = lerp(SkyCol, color, IN.fog);
	
	return float4(color,1);
}

struct GrassVertIn {
	float4 pos : POSITION;
	float3 normal : NORMAL;
	float4 color : COLOR0;
	float2 texcoords : TEXCOORD0;
};

struct GrassVertOut {
	float4 pos : POSITION;
	float2 texcoords : TEXCOORD0;
	float3 color : COLOR0;
	float fog : TEXCOORD1;
	float fade : TEXCOORD2;
};

GrassVertOut GrassVS( GrassVertIn IN ) {
	GrassVertOut OUT;
	
	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, world2);
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist ) / (FogRange-FogStart) );
	OUT.fade = 1 - (saturate( (GrassDist-dist) / (GrassDist/5) ));
	OUT.fade += 0.4;
	
	//Animate grass
	float height = clamp(IN.pos.z, 0, 100) / 100;
	
	float2 wind = WindVec * 2.5 + 0.25;
		
	float2 pos_ticks1 = worldpos.xy / 1000 + ticks;
	float2 pos_ticks2 = worldpos.xy / 750 + ticks * 2;
	float2 pos_ticks3 = worldpos.xy / 500 + ticks * 3;
	float2 pos_ticks4 = worldpos.xy / 200 + ticks * 4;
	
	worldpos.xy += ( sin(pos_ticks1) + cos(pos_ticks2) + sin(pos_ticks3) + cos(pos_ticks4) ) * height * wind * 10;
	
	//Projection
	OUT.pos = mul(worldpos, viewproj);
	
	//Lighting
	OUT.color = (SunCol * 0.25) + SunAmb;

	return OUT;
}

GrassVertOut GrassInstVS( StatInstVertIn IN ) {
	GrassVertOut OUT;
	
	//Pack instance transform
    float4x4 worldmat = float4x4(IN.m1, IN.m2, IN.m3, IN.m4);

	OUT.texcoords = IN.texcoords;
	
	//Define % to Fog
	float4 worldpos = mul(IN.pos, worldmat);
	float dist = length(worldpos.xyz - EyePos.xyz);
	OUT.fog = saturate( (FogRange - dist ) / (FogRange-FogStart) );
	OUT.fade = 1 - (saturate( (GrassDist-dist) / (GrassDist/5) ));
	OUT.fade += 0.4;
	
	//Animate grass
	float height = clamp(IN.pos.z, 0, 100) / 100;
	
	float2 wind = WindVec / 6;
	
	float2 pos_ticks1 = worldpos.xy / 1000 + ticks;
	float2 pos_ticks2 = worldpos.xy / 750 + ticks * 2;
	float2 pos_ticks3 = worldpos.xy / 500 + ticks * 3;
	float2 pos_ticks4 = worldpos.xy / 250 + ticks * 4;
	
	worldpos.xy += ( sin(pos_ticks1) + cos(pos_ticks2) + sin(pos_ticks3) + cos(pos_ticks4) ) * height * wind * 10;
	
	//Projection
	OUT.pos = mul(worldpos, viewproj);
	
	//Lighting
	OUT.color = (SunCol * 0.25) + SunAmb;

	return OUT;
}

float4 GrassPS( GrassVertOut IN ) : COLOR0 {
	float4 Result = tex2D( s3, IN.texcoords);
	clip(Result.a - IN.fade);
	
	//Lighting	
	Result.rgb *= IN.color;

	//Fogging
	Result.rgb = lerp(FogCol2, Result.rgb, IN.fog);

	return float4(Result.rgb, 1);
}

Technique T0
{
	
	//Used to render the reflected landscape
	Pass A
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		CullMode=None;

		VertexShader = compile vs_2_0 LandscapeRefVS();
		PixelShader = compile ps_2_0 LandscapeRefPS();
	}
	//Used to render the landscape
	Pass B
	{
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		AlphaBlendEnable=false;
		CullMode=CW;
		FogEnable=false;

		VertexShader = compile vs_2_0 LandscapeVertexShader();
		PixelShader = compile ps_2_0 LandscapePixelShader();
	}
	//Used to render the water in SM 2.0 mode
	Pass C
	{
		AlphaBlendEnable=false;
		CullMode=CW;
		ZFunc=Less;
		FogEnable=false;
		ZWriteEnable=true;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 ReflectionShader2();
	}
	//Used to render the water in SM 3.0 mode
	Pass D
	{
		AlphaBlendEnable=false;
		CullMode=CW;
		ZFunc=Less;
		FogEnable=false;
		ZWriteEnable=true;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 ReflectionShader2();
	}
	//Used for rendering distant statics
	Pass E
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=CW;
		
		VertexShader = compile vs_2_0 StaticVS();
		PixelShader = compile ps_2_0 StaticPS();
	}
	//Used for rendering the water from underneath
	Pass F
	{
		AlphaBlendEnable=false;
		CullMode=CCW;
		ZFunc=Less;
		FogEnable=false;
		ZWriteEnable=true;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 UnderWaterPS();
	}
	//Used for rendering water in non-reflective mode
	Pass G
	{
		AlphaBlendEnable=false;
		CullMode=CW;
		ZFunc=Less;
		FogEnable=false;
		ZWriteEnable=true;

		VertexShader = compile vs_2_0 WaterVS();
		PixelShader = compile ps_2_0 NonReflectionShader();
	}
	//Used for rendering reflected statics
	Pass H
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=CCW;

		VertexShader = compile vs_2_0 StaticReflectionVS();
		PixelShader = compile ps_2_0 StaticReflectionPS();
	}
	//Used for rendering the lowest possible land out into infinity
	Pass I
	{
		AlphaBlendEnable=false;
		CullMode=CW;
		ZFunc=Less;
		FogEnable=false;
		ZWriteEnable=true;

		VertexShader = compile vs_2_0 LowestLandVS();
		PixelShader = compile ps_2_0 LowestLandPS();
	}
	//Used for rendering distant statics with HW instancing
	Pass J
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=CW;
		
		VertexShader = compile vs_3_0 StaticInstVS();
		PixelShader = compile ps_3_0 StaticPS();
	}
	//Used for rendering reflected distant statics with HW instancing
	Pass J
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=CCW;

		VertexShader = compile vs_2_0 StaticReflectionInstVS();
		PixelShader = compile ps_2_0 StaticReflectionPS();
	}
	//Used for rendering grass
	Pass K
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_2_0 GrassVS();
		PixelShader = compile ps_2_0 GrassPS();
	}
	//Used for rendering grass with instancing
	Pass L
	{
		AlphaBlendEnable=false;
		FogEnable=false;
		CullMode=None;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;

		VertexShader = compile vs_3_0 GrassInstVS();
		PixelShader = compile ps_3_0 GrassPS();
	}
	//Used to set up the render state
	Pass Zero
	{
		AlphaBlendEnable=false;
		AlphaTestEnable=false;
		FogEnable=false;
		FogColor=<FogCol>;
		ColorVertex=false;
		Lighting=false;
		ZEnable=true;
		ZWriteEnable=true;
		ZFunc=LessEqual;
		StencilEnable=false;
		ColorWriteEnable=red|green|blue;
		CullMode=CCW;
		ShadeMode=gouraud;
		ColorOp[0]=selectarg1;
		ColorOp[1]=disable;
		AlphaOp[0]=disable;
		AlphaOp[1]=disable;
		ResultArg[0]=current;
		Texture[0]=null;
		Texture[1]=null;
		Texture[2]=null;
		Texture[3]=null;
		Texture[4]=null;
		Texture[5]=null;
		Texture[6]=null;
		Texture[7]=null;
		TexCoordIndex[0]=0;
		TexCoordIndex[1]=1;
		TexCoordIndex[2]=2;
		TexCoordIndex[3]=3;
		TexCoordIndex[4]=4;
		TexCoordIndex[5]=5;
		TexCoordIndex[6]=6;
		TexCoordIndex[7]=7;
		TextureTransformFlags[0]=0;
		TextureTransformFlags[1]=0;
		TextureTransformFlags[2]=0;
		TextureTransformFlags[3]=0;
		TextureTransformFlags[4]=0;
		TextureTransformFlags[5]=0;
		TextureTransformFlags[6]=0;
		TextureTransformFlags[7]=0;
		MagFilter[0]=Linear;
		MinFilter[0]=Linear;
		MipFilter[0]=Linear;
		Clipping=true;
	}
}